@@ -24,7 +24,7 @@ def maintain_cwd(f):
     return maintain_cwd_helper
 
 def egg_to_package(file):
-    """ Extracts the package name from an egg.
+    """ Extracts the package name from an egg::
 
         >>> egg_to_package("PyYAML-3.10-py2.7-macosx-10.7-x86_64.egg")
         ('PyYAML', '3.10-py2.7-macosx-10.7-x86_64.egg')
@@ -36,27 +36,41 @@ def egg_to_package(file):
     return (name, file[len(name)+1:])
 
 def file_to_package(file, basedir=None):
-    """ Returns the package name for a given file.
+    """ Returns the package name for a given file::
 
         >>> file_to_package("foo-1.2.3_rc1.tar.gz")
         ('foo', '1.2.3-rc1.tar.gz')
         >>> file_to_package("foo-bar-1.2.tgz")
         ('foo-bar', '1.2.tgz')
-        >>> """
-    if os.path.splitext(file)[1].lower() == ".egg":
+        >>> file_to_package("foo-bar-1.2-py27-none-any.whl")
+        ('foo-bar', '1.2-py27-none-any.whl')
+        >>> file_to_package("Cython-0.17.2-cp26-none-linux_x86_64.whl")
+        ('Cython', '0.17.2-cp26-none-linux_x86_64.whl')
+        """
+    file_ext = os.path.splitext(file)[1].lower()
+    if file_ext == ".egg":
         return egg_to_package(file)
-    split = file.rsplit("-", 1)
-    if len(split) != 2:
+
+    if file_ext == ".whl":
+        bits = file.rsplit("-", 4)
+        split = [bits[0], "-".join(bits[1:])]
+        to_safe_name = lambda x: x
+    else:
+        split = file.rsplit("-", 1)
+        to_safe_name = pkg_resources.safe_name
+
+    if len(split) != 2 or not split[1]:
         msg = "unexpected file name: %r " %(file, )
         msg += "(not in 'pkg-name-version.xxx' format"
         if basedir:
             msg += "; found in directory: %r" %(basedir)
         msg += ")"
         raise ValueError(msg)
-    return (split[0], pkg_resources.safe_name(split[1]))
 
-def archive_pip_packages(path, package_cmds):
-    pip_args = ["install", "-d", path] + package_cmds
+    return (split[0], to_safe_name(split[1]))
+
+def archive_pip_packages(pip_args, path, package_cmds):
+    pip_args = pip_args + [path] + package_cmds
     try:
         import pip
     except ImportError:
@@ -74,7 +88,8 @@ def archive_pip_packages(path, package_cmds):
     if version < "1.1":
         raise RuntimeError("pip >= 1.1 required, but %s is installed"
                            %(version, ))
-    pip.main(pip_args)
+    return pip.main(pip_args)
+
 
 def dir2pi(argv=sys.argv):
     if len(argv) != 2:
@@ -146,29 +161,42 @@ def dir2pi(argv=sys.argv):
 
 @maintain_cwd
 def pip2tgz(argv=sys.argv):
+    return pip2archive(argv)
+
+@maintain_cwd
+def pip2whl(argv=sys.argv):
+    return pip2archive(argv, use_wheels=True)
+
+def pip2archive(argv, use_wheels=False):
+    cmd_name = 'pip2whl' if use_wheels else 'pip2tgz'
+    glob_test = './*.whl' if use_wheels else './*.tar.*'
+    pip_cmd = ['wheel', '--wheel-dir'] if use_wheels else ['install', '-d']
+
     if len(argv) < 3:
         print(dedent("""
-            usage: pip2tgz OUTPUT_DIRECTORY PACKAGE_NAME ...
+            usage: {0} OUTPUT_DIRECTORY PACKAGE_NAME ...
 
             Where PACKAGE_NAMES are any names accepted by pip (ex, `foo`,
             `foo==1.2`, `-r requirements.txt`).
 
-            pip2tgz will download all packages required to install PACKAGE_NAMES and
+            {0} will download all packages required to install PACKAGE_NAMES and
             save them to sanely-named tarballs in OUTPUT_DIRECTORY.
 
             For example:
 
-                $ pip2tgz /var/www/packages/ -r requirements.txt foo==1.2 baz/
-        """))
+                $ {0} /var/www/packages/ -r requirements.txt foo==1.2 baz/
+        """.format(cmd_name)))
         return 1
 
     outdir = os.path.abspath(argv[1])
     if not os.path.exists(outdir):
         os.mkdir(outdir)
 
-    archive_pip_packages(outdir, argv[2:])
+    res = archive_pip_packages(pip_cmd, outdir, argv[2:])
+    if res != 0:
+        return res
     os.chdir(outdir)
-    num_pakages = len(glob.glob('./*.tar.*'))
+    num_pakages = len(glob.glob(glob_test))
 
     print("\nDone. %s archives currently saved in %r." %(num_pakages, argv[1]))
     return 0
